#!/bin/sh
#(c) Copyright Barry Kauler December 2013, bkhome.org
#Licence GPL3 (refer: /usr/share/doc/legal).
#131225 original install-quirky-to-partition.sh adapted by MHHP to install quirky-6.0.1 to a file system image - QUIRKMODE=8

echo
echo "This script will install Quirky 6.x to a file system image."
echo "Any kind of hard drive is acceptable, including conventional magnetic
platter or SSD (Solid State Disk -- Flash media).
The partition must ALREADY have a filesystem. For normal magnetic media,
ext2, ext3, or ext4 is recommended. For SSD, f2fs is recommended."
echo "The free space will need to be at least 2GB in size."
echo "If you do not have a suitable available partition, abort this script!"
echo
echo "The script will ask questions at each step, and will offer a choice of
partitions to install to, but of course it comes down to you to make an
intelligent choice and not erase a partition with important files on it!"
echo "This script should work when running any Linux distribution, however, it
works best when running a Puppy Linux or Puppy Linux-derivative, as Puppy has,
in my opinion, many superior utilities. However, this script will check for
existence or otherwise of those utilities and downgrade appropriately, or
abort if a missing utility is essential."
echo
echo -n "Press ENTER key to continue (CTRL-C to abort): "
read goforit

###check sanity###
if [ "`whoami`" != "root" ];then
 echo
 echo "Sorry, you need to run this script as root (administrator). Aborting."
 exit
fi
if [ $1 ];then
 echo
 echo "Nothing required on commandline, this script will ask."
fi
for AUTILITY in blkid cp dd ls mount sed stat xz
do
 if [ "`which ${AUTILITY}`" = "" ];then
  echo
  echo "Sorry, the '${AUTILITY}' utility is missing. Aborting."
  exit
 fi
done
IMGLIST="$(ls -1 ./ | grep 'quirky' | grep '.usfs.xz$' | grep -v 'empty')"
if [ "$IMGLIST" = "" ];then
 echo "The Quirky6 compressed image file should have been downloaded"
 echo "to the same folder in which you are running this script."
 echo "It will be named something like 'quirky*.usfs.xz'"
 echo "Please download it from distro.ibiblio.org/quirky/quirky6/x86"
 exit
fi
#need linux f.s....
WKGFS="$(stat -f install-quirky-to-fs_img.sh | grep -o 'Type: .*' | grep -E 'ext2|ext3|ext4|reiserfs|f2fs')"
if [ "$WKGFS" = "" ];then
 echo
 echo "Sorry, but you must be running this script in a partition with a Linux
filesystem. The 'stat' utility was used to check for ext2, ext3, ext4, reiserfs
or f2fs filesystem, however some versions of stat do not recognise f2fs.
If you are running in a f2fs filesystem, or some other Linux filesystem
(but definitely NOT a ntfs, msdos or vfat f.s.!!!), then press any printable
character on the keyboard, then ENTER key, to continue."
 echo -n "To abort, press ENTER key only: "
 read tostop
 [ "$tostop" = "" ] && exit
fi

###choose .usfs.xz file###
echo
CNT=1
echo -n "" > /tmp/quirkyimgs
for AIMG in `echo -n "$IMGLIST" | tr '\n' ' '`
do
 echo "${CNT} ${AIMG}" >> /tmp/quirkyimgs
 CNT=`expr $CNT + 1`
done
cat /tmp/quirkyimgs
echo
echo -n "Type number of image file: "
read numimg
[ "$numimg" = "" ] && exit
IMGFILEXZ="$(cat /tmp/quirkyimgs | head -n ${numimg} | tail -n 1 | cut -f 2 -d ' ')"
echo "...ok, have chosen ${IMGFILEXZ}"
##¤## MHHP, 131225
if [ `echo "${IMGFILEXZ}"` != "quirky-6.0.1.usfs.xz" ]; then
 echo -e "Sorry, install to a file system image, QUIRKMODE=8, is only implemented 
for quirky-6.0.1 as of now ...\nAs this implementation of QUIRKMODE=8 only is tested 
in version 6.0.1 of quirky (and BK is likely to continue developing at lightning speed), 
This script will only install known and tested versions of quirky into a file system image, 
i.e. QUIRKMODE=8.\n/MHHP"
 exit
fi
if [ ! -f $IMGFILEXZ ];then
 echo "sorry, file ${IMGFILEXZ} does not exist"
 exit
fi
EXT="$(echo -n $IMGFILEXZ | rev | cut -f 1 -d '.' | rev)"
if [ "$EXT" != "xz" ];then
 echo "sorry, file ${IMGFILEXZ} is not xz-compressed"
 exit
fi
echo
echo "Testing integrity of compressed file, please wait..."
xz --test $IMGFILEXZ
if [ $? -ne 0 ];then
 echo
 echo "Sorry, file appears to be corrupted. Try downloading again."
 exit
fi

#expand img file...
echo
IMGFILE="$(basename ${IMGFILEXZ} .xz)"
echo "Uncompressing ${IMGFILEXZ} to ${IMGFILE}..."
rm -f ${IMGFILE} 2>/dev/null
xz --decompress --stdout ${IMGFILEXZ} > ${IMGFILE}
sync
echo '...done'

####choose target partition###
ALLPARTS="$(cat /proc/partitions | grep -o -E 'sd[a-z][0-9]|sd[a-z][0-9][0-9]|hd[a-z][0-9]|sd[a-z][0-9][0-9]|mmcblk0p[0-9]|mmcblk0p[0-9][0-9]' | tr '\n' ' ')"
echo
echo "These are all the partitions:"
echo "$ALLPARTS"
if which probedisk > /dev/null 2>&1;then
 echo
 echo "These are all the drives:"
 probedisk
fi
if which probepart > /dev/null 2>&1;then
 echo
 echo "These are all of the partitions, showing sizes in MB:"
 probepart -m | grep -v 'none'
fi
echo
echo "You will need to choose one of these to install Quirky to. It must have a
Linux filesystem, and have more free space than 2GB.."
echo -n "Press ENTER to continue: "
read goforit
[ "$goforit" != "" ] && exit

echo
echo "Choose the partition to install to..."
CNT=1
echo -n "" > /tmp/quirkytargetparts
for AIMG in $ALLPARTS
do
 echo "${CNT} ${AIMG}" >> /tmp/quirkytargetparts
 CNT=`expr $CNT + 1`
done
cat /tmp/quirkytargetparts
echo
echo -n "Type a number from left column: "
read numimg
[ "$numimg" = "" ] && exit
TARGETPART="$(cat /tmp/quirkytargetparts | head -n ${numimg} | tail -n 1 | cut -f 2 -d ' ')" #ex: sda7
echo "...ok, have chosen ${TARGETPART}"
[ "$TARGETPART" = "" ] && exit #precaution
CHECKPART="$(echo "$ALLPARTS" | tr ' ' '\n' | grep "${TARGETPART}"'$')"
if [ ! "$CHECKPART" ];then
 echo
 echo "Error making choice, aborting."
 exit
fi

###check the target###
#check ___mounted___...
tPTN="^/dev/${TARGETPART} "
#STILL=''
#while [ "`mount | grep "$tPTN"`" == "" ];do
 #echo
 #echo "Partition ${TARGETPART} is ${STILL}not mounted."
 #echo "Mount it right now."
 #echo -n "Mount it right now, then press ENTER (CTRL-C to abort): "
 #read goforit
 #[ "$goforit" != "" ] && exit
 #STILL='still '
#done
#check f.s....
if which probepart > /dev/null 2>&1;then
 pPTN="^/dev/${TARGETPART}|"
 TARGETFS="$(probepart | grep "$pPTN" | cut -f 2 -d '|')"
else
 TARGETFS="$(blkid /dev/${TARGETPART} | grep -o 'TYPE="[^"]*' | cut -f 2 -d '"')"
fi
if [ "$(echo -n "$TARGETFS" | grep -E 'ext2|ext3|ext4|f2fs|reiserfs|xfs|btrfs')" = "" ];then
 echo
 echo "${TARGETPART} has '${TARGETFS}' filesystem, an invalid choice, aborting."
 exit
fi

FSUUID="$(blkid |grep "/dev/${TARGETPART}" | grep -o 'UUID="[^"]*' | cut -f 2 -d '"')"

###get mountpoint
TARGETMNTPNT=`mount | grep "$tPTN"|tr -s ' '|cut -d ' ' -f 3`

###write to target###
echo
echo "Now the action is starting, will soon write to partition ${TARGETPART},
which has filesystem ${TARGETFS}."
echo -n "Press ENTER to continue (CTRL-C to abort): "
read goforit
[ "$goforit" != "" ] && exit

if [ "${TARGETMNTPNT}" == "" ];then
 echo "Mounting ${TARGETMNTPNT}"
 mkdir -p /mnt/$TARGETPART
 mount -t $TARGETFS /dev/$TARGETPART /mnt/$TARGETPART
 TARGETMNTPNT=/mnt/${TARGETPART}
fi
[ "`mount | grep "$tPTN"`" != "" ] 
if [ $? -ne 0 ];then
 echo
 echo "Error, ${TARGETPART} is not mounted, aborting. Maybe you should try mounting it, or be running Puppy Linux or Puppy-derivative."
 exit
fi
echo
#echo "Do you want to erase contents of partition? Normally you will reply 'yes' to
#this, to get rid of whatever is already on the partition. However, if upgrading
#an earlier installation of Quirky, you may decline."
#echo -n "Press ENTER only to erase (or any char then ENTER to keep contents): "
#read jumpwipe
#if [ "$jumpwipe" = "" ];then
 #echo
 #echo "Erasing contents of ${TARGETPART}..."
 #rm -rf /mnt/$TARGETPART/*
 #rm -rf /mnt/$TARGETPART/.* 2> /dev/null
 #sync
#fi

##¤## MHHP, 131225
echo "Please, type the full path to the directory (in ${TARGETPART}) into which you wish to install the quirky file system image!  (CTRL-C to abort)"
read QM8DIR
QM8DIR="${QM8DIR#\/}" # Remove leading slash
if [ ! -d "${TARGETMNTPNT}/${QM8DIR}" ]; then
 echo "${TARGETMNTPNT}/${QM8DIR} does not exist, creating it ..."
 mkdir -p "${TARGETMNTPNT}/${QM8DIR}"
 if [ $? -ne 0 ];then
  echo "Couldn't create directory ${TARGETMNTPNT}/${QM8DIR}. Aborting ..."
  exit
 fi
fi
echo
echo "Which size do you want for your quirky file system image? (CTRL-C to abort)"
echo
echo -e "10     1.0 GB\n15     1.5 GB\n20     2.0 GB\n25     2.5 GB\n30     3.0 GB\n40     4.0 GB\n50     5.0 GB\n60     6.0 GB\n80     8.0 GB\n160   16.0 GB"
echo -n "Type a number from left column: "
read numsize
[ "$numsize" = "" ] && exit
[ "$numsize" = "10" ] && fsize="1024"
[ "$numsize" = "15" ] && fsize="1536"
[ "$numsize" = "20" ] && fsize="2048"
[ "$numsize" = "25" ] && fsize="2560"
[ "$numsize" = "30" ] && fsize="3072"
[ "$numsize" = "35" ] && fsize="3584"
[ "$numsize" = "40" ] && fsize="4096"
[ "$numsize" = "50" ] && fsize="5120"
[ "$numsize" = "60" ] && fsize="6144"
[ "$numsize" = "80" ] && fsize="8192"
[ "$numsize" = "160" ] && fsize="16384"
if [ "$fsize" \>= `df -BM|tr -s ' '|grep 'sdb5'|cut -d ' ' -f 4|grep -o "[0-9.]*"` ];then
 echo "Chosen file size larger than available free space on partition ${TARGETPART}. Aborting ..."
 exit
fi
echo
echo "Which of the following file systems do you want for your file system image:"
echo "ext2   ext3   ext4   f2fs"
read fs
[ "$fs" = "" ] && exit
case $fs in
 ext2|ext3|ext4|f2fs)
 ;;
 *)
  echo "Chosen file system is not one of ext2|ext3|ext4|f2fs. Aborting ..."
  exit
 ;;
esac
QM8FILENAME="quirky-6.0.1.$fs" # Yes, should provide an input dialogue ...

echo
echo "Writing Quirky6 files to ${TARGETMNTPNT}/${QM8DIR}/${QM8FILENAME}..."
mkdir -p mntpt2
mkdir -p mntpt1
mount -t squashfs -o loop,ro ${IMGFILE} mntpt2
if [ $? -ne 0 ];then
 echo
 echo "Error mounting ${IMGFILE} on mntpt2. Aborting"
 rm -f ${IMGFILE}
 exit
fi
dd if=/dev/zero of=${TARGETMNTPNT}/${QM8DIR}/${QM8FILENAME} bs=1M count="$fsize"
yes|mkfs."$fs" ${TARGETMNTPNT}/${QM8DIR}/${QM8FILENAME}

TARGETVERSION="$(grep '^DISTRO_VERSION=' mntpt2/etc/DISTRO_SPECS | cut -f 2 -d '=' | cut -f 2 -d "'" | cut -f 1 -d ' ')" #ex: 6.0.1

echo
echo "Now writing to ${TARGETMNTPNT}/${QM8DIR}/${QM8FILENAME}, please wait..."

mount -t $fs -o loop /mnt/${TARGETPART}/${QM8DIR}/${QM8FILENAME} mntpt1

###MHHP, can't follow this logic at this late hour, will continue ...      have continued...### 
cp -a -f --remove-destination mntpt2/* mntpt1/ 2> /tmp/installquirky-cp-errlog
sync
#if updating, can have a problem if want to replace a folder with a symlink. for example, got this error:
# cp: cannot overwrite directory '/mnt/sda6/usr/share/mplayer/skins' with non-directory
#3builddistro has this fix... which is a vice-versa situation...
#firstly, the vice-versa, source is a directory, target is a symlink...
if [ -s /tmp/installquirky-cp-errlog ];then #next line fixes those quote chars...
 cat /tmp/installquirky-cp-errlog | grep 'cannot overwrite non-directory' | tr '[`‘’]' "'" | cut -f 2 -d "'" |
 while read ONEDIRSYMLINK #ex: /mnt/sda6/usr/share/mplayer/skins
 do
  xONEDIRSYMLINK="$(echo -n "$ONEDIRSYMLINK" | sed -e "s%mntpt1%%")" #ex: /usr/share/mplayer/skins
  #adding that extra trailing / does the trick...
  cp -a --remove-destination mntpt${xONEDIRSYMLINK}/* mntpt1${xONEDIRSYMLINK}/
 done
fi
#secondly, which is our mplayer example, source is a symlink, target is a folder...
if [ -s /tmp/installquirky-cp-errlog ];then #next line fixes those quote chars...
 cat /tmp/installquirky-cp-errlog | grep 'cannot overwrite directory' | grep 'with non-directory' | tr '[`‘’]' "'" | cut -f 2 -d "'" |
 while read ONEDIRSYMLINK #ex: /mnt/sda6/usr/share/mplayer/skins
 do
  xONEDIRSYMLINK="$(echo -n "$ONEDIRSYMLINK" | sed -e "s%mntpt1%%")" #ex: /usr/share/mplayer/skins
  mv -f mntpt1${xONEDIRSYMLINK} mntpt1${xONEDIRSYMLINK}TEMP
  rm -rf mntpt1${xONEDIRSYMLINK}TEMP
  DIRPATH="$(dirname "mntpt1${xONEDIRSYMLINK}")"
  cp -a -f --remove-destination mntpt2${xONEDIRSYMLINK} ${DIRPATH}/
 done
fi
sync
#umount mntpt2
#fix f.s....
tPTN1="s%.*MARK_DEV1FS_LINE.*% DEV1FS='${TARGETFS}' #MARK_DEV1FS_LINE%"
#tPTN2="s%DEV1FS='f2fs'.*%DEV1FS='${TARGETFS}' #MARK_DEV1FS_LINE%"
sed -i -e "$tPTN1" mntpt1/sbin/init
#sed -i -e "$tPTN2" /mnt/${TARGETPART}/etc/rc.d/rc.sysinit

#save img file for our audit trail...
echo "Copying ${IMGFILE} to ${TARGETMNTPNT}/${QM8DIR}/${QM8FILENAME}/audit/, for audit-trail..."
mkdir -p mntpt1/audit
mv -f ${IMGFILE} mntpt1/audit/
sync
#umount ${TARGETMNTPNT}

### MHHP, should apply the following patches here ... Check the logic in the following lines!
cd mntpt1/etc/rc.d
busybox patch -i ../../../etc_rc.d_rc.shutdown.diff
busybox patch -i ../../../etc_rc.d_rc.sysinit.diff
cd ../../../

[ ! -d ${TARGETMNTPNT}/${QM8DIR}/boot ] && mkdir ${TARGETMNTPNT}/${QM8DIR}/boot
cp -a mntpt2/boot/vmlinuz ${TARGETMNTPNT}/${QM8DIR}/boot/
sync
umount mntpt1

# Making the initrd ...
echo
echo "Making the initrd ..."
dd if=/dev/zero of=initrd.2fs bs=1M count=12;yes|mkfs.ext2 initrd.2fs

mount -t ext2 -o loop initrd.2fs mntpt1
# From /sbin/init
  mkdir mntpt1/bin
  for ONEBIN in "[" "[[" ash autologinroot basename busybox cat chmod chown clear cp cut date dc dd df dirname dmesg du ed expr false find free grep head kill ln login ls lsmod mkdir mknod more mv nice ntfs-3g pidof pwd readlink rev rm rmdir sed sleep sort stat su sync tail tar touch tr true uname usleep waitmax wc which xargs
  do
   if [ -e mntpt2/bin/$ONEBIN ];then
    cp -a mntpt2/bin/$ONEBIN mntpt1/bin/
   else
    ln -s busybox mntpt1/bin/$ONEBIN
   fi
  done
  ln -snf busybox mntpt1/bin/ps
  ln -snf busybox mntpt1/bin/rdev
  ln -snf busybox mntpt1/bin/sh
  ln -snf busybox mntpt1/bin/mount
  ln -snf busybox mntpt1/bin/umount
  mkdir mntpt1/sbin
  for ONEBIN in depmod dumpe2fs e2fsck elspci fdisk freeramdisk fsck.ext2 fsck.ext3 fsck.ext4 fsck.f2fs getty guess_fstype halt hdparm hotplug2stdout hotplug2stdout_notimeout hwclock ifconfig ifplugstatus insmod klogd kmod ldconfig loadkmap losetup lsmod mingetty mount.ntfs-3g pivot_root probedisk probepart swapon #130418 131207
  do
   if [ -e mntpt2/sbin/$ONEBIN ];then
    cp -a mntpt2/sbin/$ONEBIN mntpt1/sbin/
   else
    ln -s ../bin/busybox mntpt1/sbin/$ONEBIN
   fi
  done
  #ln -snf ../bin/busybox /sbin/chroot          #must match
  ln -snf ../bin/busybox mntpt1/sbin/chroot #must match
  #ln -snf ../bin/busybox /mnt/ram0/sbin/init
  ln -snf ../bin/busybox mntpt1/sbin/poweroff
  ln -snf ../bin/busybox mntpt1/sbin/reboot
  mkdir mntpt1/etc
  #101020 no longer have /etc/modprobe.conf...
  for ONEFILE in codepage DISTRO_SPECS eventmanager fdprm floppy fontmap group gshadow host.conf hostname hosts hosts.allow hosts.deny inittab inoutrc issue keymap localtime mailcap mime.types mke2fs.conf modules.conf mtab networks nscd.conf nsswitch.conf passwd printcap protocols services shadow
  do
   cp -a mntpt2/etc/$ONEFILE mntpt1/etc/ 2>/dev/null
  done
  cp -a mntpt2/etc/modprobe.d mntpt1/etc/ 2>/dev/null #101020
  echo '/lib' > mntpt1/etc/ld.so.conf
  echo 'export PATH="/bin:/sbin"
export LD_LIBRARY_PATH="/lib"
ulimit -c 0
umask 022
export USER=root
export LOGNAME=root
export TERM=linux' > mntpt1/etc/profile
  LOCALTIME="/usr/share`readlink /etc/localtime | sed 's%^.*/zoneinfo%/zoneinfo%'`" # MHHP From running system, not pristine quirky
  LOCALTDIR="`dirname $LOCALTIME`"
  mkdir -p mntpt1/$LOCALTDIR
  cp -a $LOCALTIME mntpt1/$LOCALTDIR/
  rm -f mntpt1/etc/localtime # MHHP Timezone from running system, not pristine quirky
  ln -snf ..$LOCALTIME mntpt1/etc/localtime # MHHP Timezone from running system, not pristine quirky
  mkdir mntpt1/etc/rc.d
  ##120716 raspi: no battery-backed hardware clock, so date is wrong...
  mkdir -p mntpt1/var/local/
  ##120718 find the date...
  #HWDATE0="`hwclock --show 2>/dev/null`" #ex: Fri 29 Jun 2012 07:45:28 AM WST  -0.725833 seconds
  #if [ "$HWDATE0" = "" ];then
   #if [ -f /var/local/shutdown_date_saved ];then #see /etc/rc.d/rc.shutdown
    #HWDATE1="`cat /var/local/shutdown_date_saved`"
   #else
    ##either of these formats can set the date: '29 JUN 2012 10:00:00' '2012-06-28 16:20:08'
    ##120718 busybox date says '29 JUN 2012 10:00:00' invalid format
    #HWDATE1="`stat --format=%z /etc/DISTRO_SPECS | cut -f 1 -d '.' | cut -f 1,2 -d ' '`" #creation date of build. ex: '2012-06-28 16:20:08'
   #fi
   #HWDATE0="$HWDATE1"
  #fi
  echo "#!/bin/sh
echo 'Now running in ramdisk ...'
" > mntpt1/etc/rc.d/rc.sysinit
  chmod 755 mntpt1/etc/rc.d/rc.sysinit
  #cp -a /dev /mnt/ram0/
  mkdir mntpt1/dev
  mkdir mntpt1/lib
  cp -a mntpt2/lib/lib*.so* mntpt1/lib/
  cp -a mntpt2/lib/ld*.so* mntpt1/lib/
  cp -a mntpt2/usr/lib/libf2fs.so* mntpt1/lib/ 2>/dev/null #131207
  cp -a mntpt2/lib/libf2fs.so* mntpt1/lib/ 2>/dev/null #131209
  [ ! -e mntpt1/lib/libgcc_s.so.1 ] && cp -a -f --remove-destination mntpt2/usr/lib/libgcc_s.so.* mntpt1/lib/ 2>/dev/null #120516 gentoo fix.
  cp -a mntpt2/lib/e2initrd_helper mntpt1/lib/ 2>/dev/null #dunno what this is.
  mkdir mntpt1/tmp
  mkdir -p mntpt1/var/run
  mkdir mntpt1/proc
  mkdir -p mntpt1/usr/share
  cp -a mntpt2/usr/share/terminfo mntpt1/usr/share/

cd mntpt1/sbin
busybox patch -i ../../initrd__sbin_init.diff
cd ../../
chmod 755 mntpt1/sbin/init

###End Making the initrd ...
sync
umount mntpt2
umount mntpt1
sync
cp -a initrd.2fs ${TARGETMNTPNT}/${QM8DIR}/boot/initrd

echo
echo "That's it, done!"

echo
echo "You will need to configure LILO, GRUB, or whatever boot manager you
use, to boot from this partition. For example, for GRUB4DOS, booting off
partition ${TARGETPART}, put this in menu.lst:

title Quirky ${TARGETVERSION} (QUIRKMODE=8, file system image install on ${TARGETPART})
  uuid ${FSUUID}
  kernel /${QM8DIR}/boot/vmlinuz root=/dev/ram0 qfsimg=$TARGETFS,/dev/UUID=${FSUUID}/${QM8DIR}/${QM8FILENAME}
  initrd /${QM8DIR}/boot/initrd

Note: The UUID of the filesystem was obtained by running 'blkid'.
Note: Unlike Puppy Linux, Quirky only recognises two Quirky-specific kernel
      boot parameters, 'qfix=nox' and 'qfix=fsck'."

###END###
